/**
 * @description Denne filen inneholder AdminUI og Tile komponentene
 * 
 * @file AdminUI.js
 */
import { useState, useEffect, Component, useContext, useRef } from 'react';
import { renderMatches, useLocation, useParams } from 'react-router-dom';
import 'chart.js/auto'
import Sidebar from './Sidebar'
import { Pie, Bar, Line } from "react-chartjs-2";
import './AdminUI.css';
import { Modal, Button, FormGroup, Form } from "react-bootstrap";
import axios from 'axios';
import { checkToken } from './tokenHandling';
//import { response } from 'express';

/**
 * @brief Tile-class component
 * 
 * @param props properties of the tile. JSON object
 */
export class Tile extends Component {
    constructor(props) {
        super(props);
        this.state = { 
            response: {}, 
            displaydata: { 
                labels: [],
                datasets: [], ///< Variable to hold datasets for 
            } 
        };
    }

    /**
     * @brief Parses the response and sets datasets/labels in accordance to what the chart requires
     * @returns datasets and labels for chartjs
     */
    parseResponse()
    {
        //If the response have no keys, exit
        if(Object.keys(this.state.response).length === 0)
            return;
        

        //Renders the different charts based on the display mode of the tile
        switch(this.props.data.display_mode)
        {
            case 'bar-x-by-y':
                // Not yet implemented
                break;
            case 'line-x-vs-y':
                // Not yet implemented
                break;
            
            /**
             * Requires a chosen datapoint by the user.
             * 
             * Sum every occurrence of the datapoint
             */
            case 'pie-x-by-y':
                let sums = [];
                let labels = [];
                
                let datachoice = "dest_port"; ///< Test choice

                // Add all unique x points to labels
                this.state.response.map((e) => {
                    if(labels.find(e[datachoice]))
                        labels.push(e[datachoice])
                });

                // Sum up all occurences of datapoint x
                for(let i = 0; i < labels.length; i++)
                {
                    sums.push(0);
                    this.state.response.map((e) => {
                        if(e[datachoice] === labels[i])
                            sums[i] += 1;
                    });
                }
                this.state.displaydata = {  datasets: sums,
                                            labels: labels };
                break;
        }
    }

    /**
     * @brief Displays the data of the tile
     * @returns A react-chartjs component
     */
    displayData()
    {
        switch(this.props.data.display_mode)
        {
            case 'bar-x-by-y':
                break;
            case 'line-x-vs-y':
                break;
            case 'pie-x-by-y':
                return(<Pie data={this.state.displaydata}/>);
        }
    }

    /**
     * 
     * @returns A function to be called at set intervals. When component gets unmounted, function will also be released
     */
    componentDidMount() {
        if (this.props.createNew) return;
        let refreshRate = this.props.data.refresh_rate;
        if(refreshRate < 20000) refreshRate = 20000;
        console.log(this.props.data.api_call)
        const api_interval = setInterval(() => {
            axios
            .get(this.props.data.api_call)
            .then((res) => { 
                this.state.response = JSON.parse(res.data);
            })
            .catch((err) => console.log(err))

            this.parseResponse();
          }, refreshRate);

          return () => clearInterval(api_interval)
    }

    /**
     * @returns JSX code that renders the tile
     */
    render() { 
        if(this.props.createNew)
        {
            return (
            <div className="tile" key={-1}>
                <h3>Create new</h3>
                <button className="createNew" onClick={this.props.handleCreateNewCB}>
                    Create new
                </button>
            </div>
            );
        }

        return ( 
            <div className="tile">
                <div className="tile-header">
                    <h3>{this.props.data.title}</h3>
                    <button className="delete" id={this.props.id} onClick={this.props.deleteTileCB}>
                        X
                    </button>
                    <button className="Edit" id={this.props.id} onClick={this.props.updateTileCB}>
                        Edit
                    </button>
                </div>
                {this.displayData()}
            </div>
        );
    }

    
}


/**
 * 
 * @param {*} props JSON object to set the initial properties of the admin object.
 * @returns JSX code to display the adminUI
 */
const AdminUI = (props) => {

    const [tiles, setTiles] = useState([]);                 ///< Variable to hold the tiles
    const [modal, setModal] = useState(false);              ///< Variable to hold which modal to display

    const [title, setTitle] = useState('');                 ///< Title of a current tile
    const [displayMode, setDisplayMode] = useState('');     ///< What chart the data is to be represented by
    const [order, setOrder] = useState(0);                  ///< Sets where the tile should appear. Lower order tiles come before higher order tiles
    const [apiCall, setApiCall] = useState('');             ///< Which API the data is to be collected from
    const [activeTile, setActiveTile] = useState(-1);       ///< ID of the active tile
    const [apiFields, setApiFields] = useState([]);         ///< Used for what data to show
    const [refreshRate, setRefresh] = useState(1)           ///< How often the tile ought to  refresh (Only used when creating a tile)
    
    // Opens and closes modal component
    const modalOpen = () => { setApiFields([]); setModal(true); };
    const modalClose = () => { setApiFields([]); setModal(false) };

    // Modal properties
    const [modalState, setModalState] = useState('createnew');

    const params = useParams();

    /**
     * @abstract This function runs on load and rerender. Updates the tiles from the database
     */
    useEffect( () => {
        axios
        .get('http://localhost:8000/api/tiles/')
        .then((res) => { setTiles(res.data) })
        .catch((err) => console.log(err))
    }, []);

    /**
     * Updates the tiles from the database
     */
    const updateDataFromApi = () => {
        axios
        .get('http://localhost:8000/api/tiles/')
        .then((res) => { setTiles(res.data);   })
        .catch((err) => console.log(err))
    }

    /**
     * @abstract Deletes a tile
     * @param {e} e Event
     */
    const deleteTile = (e) => {
        let deleteTarget = "http://localhost:8000/api/tiles/" + e.target.id + '/';
        axios
        .delete(deleteTarget, { data: {id: e.target.id}})
        .then((res) => { updateDataFromApi()})
        .catch((err) => console.log(err) )
    }

    /**
     * @abstract Sets the clicked tile to the active tile.
     * @param {e} e The on-click event
     */
    const updateTile = (e) => {
        setModalState('edit');
        setActiveTile(e.target.id);
        setTitle(tiles.find((el) => el.id == e.target.id).title);
        setDisplayMode(tiles.find((el) => el.id == e.target.id).display_mode);
        setApiCall(tiles.find((el) => el.id == e.target.id).api_call);
        setOrder(tiles.find((el) => el.id == e.target.id).order);
        setRefresh(tiles.find((el) => el.id == e.target.id).refresh_rate);
        setActiveTile(e.target.id);

        modalOpen();
    }

    /**
     * @abstract Filters out all the tiles that do not belong to the window and sorts them based on their order
     * @returns an array of tiles to be displayed
     */
    const renderTiles = () => {
        const data = tiles.filter(e => e.window_id == params.id);
        
        
        return (data.sort((a, b) => a.order-b.order)).map( (tile) => (
            <Tile data={tile} key={tile.id} deleteTileCB={deleteTile.bind()} updateTileCB={updateTile.bind()}/>
        ));
    }

    /**
     * @brief changes tile states to reflect changes made in the modal
     * @param {*} e on-click event
     */
    const handleInputChange = (e) => {
        if(e.target.id === 'title')
            setTitle(e.target.value);
        else if(e.target.id === 'display-mode')
            setDisplayMode(e.target.value);
        else if(e.target.id === 'api-call')
            setApiCall(e.target.value);
        else if(e.target.id === 'order')
            setOrder(e.target.value);
        else if(e.target.id === 'refresh-rate')
            setRefresh(e.target.value);
        else {
            let fields = apiFields;
            let field = {...fields[e.target.id-1]};
            if(field.isChecked) field.isChecked = false;
            else field.isChecked = true;
            fields[e.target.id-1] = field;
            setApiFields(fields);
        }
    }

    // Changes modal state to createnew and opens the modal 
    const handleCreateNew = () => {
        setModalState('createnew');

        modalOpen();
    }

    /**
     * @abstract Creates a new tile from the current tile-states. Sends a post request to the database
     */
    const handleCreateNewTile = async () => {
        const datapoints = JSON.stringify(apiFields);
        const newTile = {
            window_id: params.id,
            title: title,
            data: datapoints,
            order: order,
            api_call: apiCall,
            display_mode: displayMode,
            refresh_rate: refreshRate,
            options: '',
        };

        axios.post('http://localhost:8000/api/tiles/', newTile)
        .then((res) => { updateDataFromApi(); modalClose(); })
        .catch(err => { console.log('') })
    }

    /**
     * @abstract Creates a JSON-object with the current tile states and sends a put request to the database with the new data for the tile.
     */
    const handleEditTile = async () => {
        const editTile = {
            id: activeTile,
            window_id: params.id,
            title: title,
            order: order,
            api_call: apiCall,
            display_mode: displayMode,
            refresh_rate: refreshRate,
            options: '',
        }

        
        axios
        .put('http://localhost:8000/api/tiles/' + activeTile + '/', editTile)
        .then((res) => { setTiles([]); updateDataFromApi(); modalClose();})
        .catch((err) => console.log(err) )
    }

    /**
     * @abstract Makes a single get request to an API location to see what fields are in the API
     */
    const fetchApiFields = () => {
        axios
        .get(apiCall)
        .then((res) => {
            let jres = JSON.parse(res.data);
            let ret = [];
            let tF = Object.keys(jres[0]);
            tF.map((e) => (
                ret.push({'field': e, 'isChecked': false})
            ));
            setApiFields(ret);
         })
        .catch((err) => console.log(err))
    }


    /**
     * @abstract Displays a popup that 
     * @param {*} modalState Which modal to display, only 2 options: createnew or edit
     * @returns JSX code that displays the modal
     */
    const customModal = (modalState) => {
        switch(modalState){
            
            case 'createnew':
                return ( 
                    <Modal show={modal} onHide={modalClose}>
                    <Modal.Header closeButton>
                    <Modal.Title>Create new tile</Modal.Title>
                    </Modal.Header>
                    <Modal.Body>
                        <div className="form-group">
                            <label htmlFor="title">Title</label>
                            <input type="text" className="form-control" id='title' onChange={handleInputChange}/>
                        </div>
                        <div className="form-group">
                            <label htmlFor="display-mode">Display mode: </label>
                            <select className="custom-select custom-select-sm" id='display-mode' onChange={handleInputChange}>
                                <option defaultValue={displayMode}>Open this select menu</option>
                                <option value="pie">Pie</option>
                                <option value="bar">Bar</option>
                                <option value="line">Line</option>
                            </select>
                        </div>
                        <div className="form-group">
                            <label htmlFor="api-call">Api call</label>
                            <input type="text" className="form-control" id='api-call' onChange={handleInputChange}/>
                            <Button variant="primary" onClick={fetchApiFields}>Fetch</Button>
                        </div>
                        <div className="form-group">
                            {apiFields.map((e, idx) => (
                                    <Form.Check
                                        label={e.field}
                                        name="group1"
                                        type='checkbox'
                                        id={idx+1}
                                        onChange={handleInputChange}
                                    />
                            ))}
                        </div>
                        <div className="form-group">
                            <label htmlFor="order">Order</label>
                            <input type="number" className="form-control" id='order' onChange={handleInputChange}/>
                        </div>
                        <div className="form-group">
                            <label htmlFor="refresh-rate">Refresh rate</label>
                            <input type="text" className="form-control" id='refresh-rate' onChange={handleInputChange}/>
                        </div>
                    </Modal.Body>
                    <Modal.Footer>
                        <Button variant="primary" onClick={handleCreateNewTile}>
                            Add new tile
                        </Button>
                    </Modal.Footer>
                </Modal>
                );
                break;
            case 'edit':
                return (
                    <Modal show={modal} onHide={modalClose}>
                        <Modal.Header closeButton>
                            <Modal.Title>Edit tile</Modal.Title>
                        </Modal.Header>
                        <Modal.Body>
                            <div className="form-group">
                                <label htmlFor="title">Title</label>
                                <input type="text" className="form-control" id='title' defaultValue={title} onChange={handleInputChange}/>
                            </div>
                            <div className="form-group">
                                <label htmlFor="display-mode">Display mode: </label>
                                <select className="custom-select custom-select-sm" id='display-mode' onChange={handleInputChange}>
                                    <option defaultValue={displayMode}>Current mode: {displayMode}</option>
                                    <option value="pie">Pie</option>
                                    <option value="bar">Bar</option>
                                    <option value="line">Line</option>
                                </select>
                            </div>
                            <div className="form-group">
                                <label htmlFor="api-call">Api call</label>
                                <input type="text" className="form-control" id='api-call' defaultValue={apiCall} onChange={handleInputChange}/>
                            </div>
                            <div className="form-group">
                                <label htmlFor="order">Order</label>
                            <input type="number" className="form-control" id='order' defaultValue={order} onChange={handleInputChange}/>
                            </div>
                            <div className="form-group">
                                <label htmlFor="refresh-rate">Refresh rate</label>
                                <input type="text" className="form-control" id='refresh-rate' defaultValue={refreshRate} onChange={handleInputChange}/>
                            </div>
                        </Modal.Body>
                        <Modal.Footer>
                            <Button variant="primary" onClick={handleEditTile}>
                                Update
                            </Button>
                        </Modal.Footer>
                    </Modal>
                );
                break;
        }
        
    }

    return ( 
        <div className="AdminUI">
            <div className="Sidebar"><Sidebar/></div>
            <div className="Tiles">{ renderTiles() }
                <Tile createNew={true} handleCreateNewCB={handleCreateNew} />
                {customModal(modalState)}
            </div>
        </div>
        );
}

export default AdminUI;
