/**
 * @description Definerer hvordan sidebaren skal fungere.
 * 
 * @file Sidebar.js
 */

import { ProSidebar, Menu, MenuItem, SubMenu } from 'react-pro-sidebar';
import 'react-pro-sidebar/dist/css/styles.css';
import './Sidebar.css';
import { BrowserRouter as Link} from 'react-router-dom'
import React, { useEffect, useState } from 'react';
import { FaFolder, FaFile } from 'react-icons/fa';
import { Modal, Button, FormGroup, Form, FormCheck} from "react-bootstrap";
import axios from 'axios';

/**
 * Definerer sidebaren og hvilke mapper og vinduer den skal vise, bestemmer 
 * også hvordan den ulike funksjonaliteten til sidebaren skal fungere.
 * 
 * @returns JSX kode for hvordan sidebaren skal se ut
 */
const Sidebar = () => {

    const [directoryList, setDirectoryList] = useState(null); ///< Listen over mapper og vinduer
    const [modal, setModal] = useState(false); ///< En modal komponent
    
    // Variables used for creating a new sidebar object
    const [activeID, setActiveID] = useState(-1); ///< IDen til mappen er tilen som skal vises
    const [title, setTitle] = useState('');  ///< Hvilken tile som skal vises
    const [directory, setDirectory] = useState(false); ///< Hvilken mappe som skal vises

    /**
     * Fremstiller en enkelt gjenstand i listen over mapper og vinduer
     * 
     * @param {*} item Mappen eller vinduet som skal fremstilles
     * @returns JSX kode for hvordan mappen eller vinduet skal fremstilles
     */
    const renderItem = (item) => {
        if(item.directory)
        {
            return(
                <SubMenu 
                    title={item.title} 
                    key={item.id} 
                    icon={<FaFolder />}
                >
                    <div className="menubuttons">
                        <button className="create-window" id={item.id} onClick={handleClickNewUnderFolder}>Create new window</button>
                        <button className="delete-window" id={item.id} onClick={handleClickDeleteUnderItem}>Delete</button>
                    </div>
                    {renderChildren(item)}
                </SubMenu>
            );
        }
        else
        {
            return(
                <>
                <MenuItem key={item.id} icon={<FaFile />}> 
                    {item.title}
                    <Link to={"/admin/" + item.id}></Link>

                </MenuItem>
                <button className="delete-window" id={item.id} onClick={handleClickDeleteUnderItem}>Delete</button>
                <form>
  <             button formaction={"/window/" + item.id}>Present</button>
                </form>
                </>
            );
        }
    }

    /**
     * Fremstiller barnene til en mappe
     * 
     * @param {*} parent Hva foreldre mappen er
     * @returns Fremstiller barnet og kaller på renderItem funksjonen
     */
    const renderChildren = (parent) => {
        return(parent.children.map((e) => <>{renderItem(e)}</>));
    }

    /**
     * Fremstiller hele listen med mapper og vinduer.
     * 
     * @param {*} list Hvilken liste som skal fremstilles
     * @returns JSX kode for hvordan mappen eller vinduet skal fremstilles.
     */
    const renderDirectoryList = (list) => {
        if(list === null)
            return(<></>);
        
        let roots = list.filter(e => e.parent === null);
        
        return(roots.map((e) => 
            { if(e.directory == true) {
                return (
                <SubMenu 
                    title={e.title} 
                    key={e.id} 
                    icon={<FaFolder />}
                >
                    <div className="menubuttons">
                        <button className="create-window" id={e.id} onClick={handleClickNewUnderFolder}>Create new window</button>
                        <button className="delete-window" id={e.id} onClick={handleClickDeleteUnderItem}>Delete</button>
                    </div>
                    {renderChildren(e)}
                </SubMenu>)
            } else {
                return (<>
                        <MenuItem key={e.id} icon={<FaFile />}>
                            {e.title}
                            <Link to={"/admin/" + e.id}></Link>
                        </MenuItem>
                        <button className="delete-window" id={e.id} onClick={handleClickDeleteUnderItem}>Delete</button>
                        <form>
  <                         button className="present-window" formaction={"/window/" + e.id}>Present</button>
                        </form>
                        </>)
            }}
            
        ));
    }

    /**
     * Parserer en liste med mapper og vinduer slik at den blir enklere å fremstille
     * 
     * @param {*} data Listen over mapper og vinduer som skal parseres
     * @returns Den ferdig parserte listen
     */
    const parseDirectorylist = (data) =>
    {
        let res = data;
        
        res.map( e => e.children = []);
        for(let i = 0; i < res.length; i++)
        {
            if(res[i].parent !== null)
            {
                let pid = res[i].parent;
                let p = res.findIndex(e => pid == e.id)
                res[p].children.push(res[i]);
            }
        }

        return res;
    }

    /**
     * Henter initielt inn alle vinduene og mappene
     */
    useEffect( () => {
        axios
        .get('http://localhost:8000/api/windows/')
        .then((res) => { setDirectoryList(parseDirectorylist(res.data))})
        .catch((err) => console.log(err));
    }, []);

    /**
     * Henter inn alle vinduene og mappene
     */
    const updateData = () => {
        axios
        .get('http://localhost:8000/api/windows/')
        .then((res) => { setDirectoryList(parseDirectorylist(res.data))})
        .catch((err) => console.log(err));
    }
    
    /**
     * Lukker igjen modal komponenten
     */
    const modalClose = () => {
        setDirectory(false);
        setModal(false);
    }

    /**
     * Setter den active IDen til den mappen som blir klikket på og
     * åpner en modal komponent
     * 
     * @param {*} e Mappen som IDen skal bli satt til
     */
    const handleClickNewUnderFolder = (e) => {
        setActiveID(e.target.id);
        setModal(true);
    }

    /**
     * Setter den active IDen til det vinduet som blir klikket på og
     * henter inne all informasjonen til den vinduet
     * 
     * @param {*} e Vinduet som IDen skal bli satt til
     */
    const handleClickDeleteUnderItem = (e) => {
        setActiveID(e.target.id);
        axios
        .delete('http://localhost:8000/api/windows/' + e.target.id)
        .then((res) => { updateData(); })
    }

    /**
     * Setter aktiv tittel eller directory til en state
     * 
     * @param {*} e Objektet som skal bli satt i en state
     */
    const handleInputChange = (e) => {
        if(e.target.id === 'title')
            setTitle(e.target.value);
        if(e.target.id === 'directory')
            setDirectory(!directory);
    }
    
    /**
     * Lager et nytt objekt i sidebaren, lagrer så denne i databasen
     */
    const createNewSidebarObject = () => {
        //Liste over variabler som skal bli pushet til databasen
        let newSidebarObject = {
            parent: activeID,
            title: title,
            directory: directory,
        };
        axios
        .post('http://localhost:8000/api/windows/', newSidebarObject)
        .then((res) => { updateData(); })
        modalClose();
    }

    /**
     * Komponten som bli returnert, hele sidebaren med alle vinduer og mapper
     */
    return ( 
        <div className="sidebar">
            <ProSidebar
                breakPoint='xs'
                image='https://raw.githubusercontent.com/azouaoui-med/react-pro-sidebar/master/demo/src/assets/bg1.jpg'
            >
                <Menu
                >
                    <button className="create-window" id={null} onClick={handleClickNewUnderFolder}>Create new window</button>
                    {renderDirectoryList(directoryList)}
                </Menu>
            </ProSidebar>
            <Modal show={modal} onHide={modalClose}>
                <Modal.Header closeButton>
                    <Modal.Title>Create new sidebar object</Modal.Title>
                </Modal.Header>
                <Modal.Body>
                    <FormGroup>
                        <label htmlFor="title">Title</label>
                        <input type="text" className="form-control" id='title' onChange={handleInputChange}/>
                    </FormGroup>
                    <FormGroup>
                        <FormCheck type="checkbox" label='Directory' id='directory' onChange={handleInputChange}/>
                    </FormGroup>

                </Modal.Body>
                <Modal.Footer>
                    <Button variant='secondary'>
                        Something interesting
                    </Button>
                    <Button variant="primary" onClick={createNewSidebarObject}>
                        Add new object
                    </Button>
                </Modal.Footer>
            </Modal>
        </div>
    );
}

export default Sidebar;
